/*
** Module   : NativeSequence.java
** Abstract : Static class that handles sequence functions and statements 
**             from Progress / OpenEdge 10.x.
**
** Copyright (c) 2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 OM  20121116          Created initial version
*/

package com.goldencode.p2j.persist;

import java.sql.*;

import com.goldencode.p2j.persist.dialect.P2JDialect;
import com.goldencode.p2j.util.integer;

/**
 * Implementation of sequences using native SQL queries.
 */
public class NativeSequence extends Sequence
{
   /** Protected getter for current value */
   protected integer _currentValue(String seqName, String ldbName)
   {
      //Database database = ConnectionManager.get().getDatabase(ldbName);
      //P2JDialect dialect = (P2JDialect) 
      //      PersistenceFactory.getInstance(database).getDialect();

      if (ldbName == null)
      {
         ldbName = _getDatabaseForSequence(seqName);
      }

      Persistence persistence = ConnectionManager.getPersistence(ldbName);
      P2JDialect dialect = (P2JDialect) persistence.getDialect();
      String strQuery = dialect.getSequenceCurrValString(seqName);
      integer ret = new integer(); // unknown result, (TODO: promote it to int64)

      try
      {
         ResultSet rs = persistence.executeSQLQuery(strQuery);
         if (rs.next())
         {
            ret = new integer(rs.getLong(1));
         }
      }
      catch (PersistenceException pe)
      {
         pe.printStackTrace();
      }
      catch (SQLException sqle)
      {
         sqle.printStackTrace();
      }

      return ret;
   }

   /**
    * Protected function that computes and returns the next value of the
    * sequence.
    */
   protected integer _nextValue(String seqName, String ldbName)
   {
      if (ldbName == null)
      {
         ldbName = _getDatabaseForSequence(seqName);
      }

      Persistence persistence = ConnectionManager.getPersistence(ldbName);
      P2JDialect dialect = (P2JDialect) persistence.getDialect();
      String strQuery = dialect.getSequenceNextValString(seqName);
      integer ret = new integer(); // unknown result, (TODO: promote it to int64)

      try
      {
         ResultSet rs = persistence.executeSQLQuery(strQuery);
         if (rs.next())
         {
            ret = new integer(rs.getLong(1));
         }
      }
      catch (PersistenceException pe)
      {
         pe.printStackTrace();
      }
      catch (SQLException sqle)
      {
         sqle.printStackTrace();
      }

      return ret;
   }

   /**
    * Protected setter for the sequence value.
    */
   protected void _setValue(String seqName, String ldbName, long val)
   {
      if (ldbName == null)
      {
         ldbName = _getDatabaseForSequence(seqName);
      }

      Persistence persistence = ConnectionManager.getPersistence(ldbName);
      P2JDialect dialect = (P2JDialect) persistence.getDialect();
      String strQuery = dialect.getSequenceSetValString(seqName, val);

      try
      {
         persistence.executeSQLQuery(strQuery);
      }
      catch (PersistenceException pe)
      {
         pe.printStackTrace();
      }
   }

}