/*
** Module   : TemporarySequence.java
** Abstract : Static class that handles sequence functions and statements 
**             from Progress / OpenEdge 10.x.
**
** Copyright (c) 2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 OM  20121116          Created initial version
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.integer;

public class TemporarySequence extends Sequence
{
   /** Protected accesor for current value */
   protected integer _currentValue(String seqName, String ldbName)
   {
      return isUnknown ? new integer() : new integer(crtValue);
   }

   /**
    * Protected function that computes and returns the next value of the
    * sequence.
    */
   protected integer _nextValue(String seqName, String ldbName)
   {
      // handle exceptions first
      if (isUnknown || step == 0)
      {
         return new integer();
      }

      if (step < 0 && limit > crtValue)
      {
         isUnknown = true;
         return new integer();
      }

      if (step > 0 && limit < crtValue)
      {
         isUnknown = true;
         return new integer();
      }

      crtValue += step;
      // check overflow:
      if ((step > 0 && crtValue > limit) || (step > 0 && crtValue > limit))
      {
         if (cycling)
         {
            crtValue = initialValue;
         }
         else
         {
            isUnknown = true;
            return new integer();
         }
      }

      return _currentValue(seqName, ldbName);
   }

   /**
    * Private setter for the sequence value.
    */
   protected void _setValue(String seqName, String ldbName, long val)
   {
      crtValue = val;
      isUnknown = false;
   }

   /** The initial value of the sequence */
   private long initialValue = 0;

   /** The increment. Can be positive or negative. 0 is illegal */
   private long step = 1;

   /** Cycling property. A cycling sequence will warp around to initial */
   private boolean cycling = false;

   /** Current value of the sequence. */
   private long crtValue = 0;

   /** The maximum/minimum value of the sequence. If current value passes over */
   private long limit = 1000;

   /** Flag for non-cycling sequences that passed the limit */
   private boolean isUnknown = false;

   /** Logical database name */
   private String ldbName = null;

   /** Sequence's name */
   private String name = null;
}
